/*
   NatNetTypes defines the public, common data structures and types
   used when working with NatNetServer and NatNetClient objects.
*/

#pragma once

#ifdef NATNETLIB_EXPORTS
#define DLLCLASS __declspec( dllexport )
#else
#define DLLCLASS __declspec( dllimport )
#endif

// version
const unsigned char NatNetVersion[4] = {1,0,0,0};

// client/server message ids
#define NAT_PING					0 
#define NAT_PINGRESPONSE			1
#define NAT_REQUEST					2
#define NAT_RESPONSE				3
#define NAT_REQUEST_MODELDEF		4
#define NAT_MODELDEF				5
#define NAT_REQUEST_FRAMEOFDATA		6
#define NAT_FRAMEOFDATA				7
#define NAT_MESSAGESTRING			8
#define NAT_UNRECOGNIZED_REQUEST	100

// common defines
#define UNDEFINED					9999.999f
#define MAX_MODELS					20
#define MAX_RIGIDBODIES				1000

// data types

typedef double SegmentData[7];	// posX, posY, posZ, rotX, rotY, rotZ, bonelength
typedef float  MarkerData[3];	// posX, posY, posZ

typedef enum Verbosity
{
    Verbosity_None=0,   // no mesage
    Verbosity_Error,    // error message
    Verbosity_Warning,  // warning message
    Verbosity_Info,     // information message
    Verbosity_Debug,    // debug message
}
Verbosity;


typedef struct sSender
{
    char          szName[128];
    unsigned char Version[4];

} sSender;

// packet
typedef struct sPacket
{
    unsigned short iMessage;
    unsigned short nDataBytes;
    union
    {
        unsigned char  cData[2000];
        char           szData[2000];
        unsigned long  lData[500];
        float          fData[500];
        sSender        Sender;
    } Data;

} sPacket;

// host descriptor
typedef struct HostInfo
{
    int FoundHost;							// have talked to Host
    char szHostComputerName[128];			// host computer name
    unsigned char HostComputerAddress[4];	// host IP address
    char szHostApp[128];					// Name of host app 
    unsigned char HostAppVersion[4];		// Version of host app

} HostInfo;

// Skeleton hierarchy description
typedef struct sMocapModelHierarchy
{
    int nSegments;						// Number of segments
    char** szSegmentNames;				// Array of segment names
    int* iParents;						// Array of segment parents

} sMocapModelHierarchy;

// Mocap model definition (markerset and skeleton hierarchy)
typedef struct sMocapModel
{
    char* szName;						// Name of the object
    int nMarkers;						// Number of markers
    char** szMarkerNames;				// Array of marker names
    sMocapModelHierarchy Hierarchy;		// model/skeleton hierarchy

} sMocapModel;

// Mocap streaming data description (MocapModel +)
typedef struct sMocapModels
{
    int nMocapModels;						// Number of models
    sMocapModel arrMocapModels[MAX_MODELS];	// The definition of each body
    void* reserved;							// reserved

} sMocapModels;

// Single frame of mocap marker and joint angle data for a single Mocap Model
typedef struct sMocapData
{
    char szName[128];					// MocapModel name
    int nMarkers;						// Number of markers
    MarkerData* Markers;				// 2D Array of marker data ( [nMarkers][3] )
    int nSegments;						// Number of segments
    SegmentData* Segments;				// 2D array of segment data ( [nSegments][7] )

} sMocapData;

// Data for a single rigid body
typedef struct sRigidBodyData
{
    int ID;								// ID
    float x, y, z;						// Position
    float qx, qy, qz, qw;				// Orientation
    int nMarkers;						// number of markers associated with this rigid body
    MarkerData* Markers;				// marker data ( [nMarkers][3] )
};

// Single frame of mocap data (for n MocapModels)
typedef struct sFrameOfMocapData
{
    int iFrame;									// app frame number
    int nMocapModels;							// The bodies should match the descriptions
    sMocapData MocapData[MAX_MODELS];			// The data for each body.
    int nOtherMarkers;							// Number of unrecognized markers
    MarkerData* OtherMarkers;					// The unrecognized markers.
    float fLatency;								// time delta between capture and send
	int nRigidBodies;							// Number of rigid bodies
	sRigidBodyData RigidBodies[MAX_RIGIDBODIES];// Data for each rigid body

} sFrameOfMocapData;


