/*

SampleClient.cpp

This program connects to a NatNet server, receives a data stream, and writes that data stream
to an ascii file.  The purpose is to illustrate using the NatNetClient class.

Usage [optional]:

	SampleClient [ServerIP]

	[ServerIP]			IP address of the server (e.g. 192.168.0.107) ( defaults to local machine)
	[OutputFilename]	Name of points file (pts) to write out.  defaults to client-output.pts

*/

#include <stdio.h>
#include <tchar.h>
#include <conio.h>
#include <winsock2.h>

#include "NatNetTypes.h"
#include "NatNetClient.h"
#include "NatNetHelper.h"

void __cdecl DataHandler(sFrameOfMocapData* data);			// receives data from the server
void __cdecl MessageHandler(int msgType, char* msg);		// receives NatNet error mesages

NatNetClient theClient;
FILE* fp;

int _tmain(int argc, _TCHAR* argv[])
{
	int retCode;

	// Initialize Win32 networking
	retCode = NatNetHelper::InitializeWinsock();
	if(retCode !=0 )
	{
		char s[1024];
		NatNetHelper::GetSocketError(retCode, s);
		printf("Winsock error: %s. Exiting.", s);
		return 1;
	}

	// Set callback handlers
	NatNetHelper::SetErrorMsgHandlerFunc(MessageHandler);
	NatNetHelper::User_VerbosityLevel = Verbosity_None;
    theClient.SetDataHandlerFunc( DataHandler );	// this function will receive data from the server

	// Connect to NatNet server
	char szMyIPAddress[128] = "";
	char szServerIPAddress[128] = "";
	if(argc>1)
	{
		strcpy(szServerIPAddress, argv[1]);	// specified on command line
		printf("Connecting to server at %s...", szServerIPAddress);
	}
	else
	{
		strcpy(szServerIPAddress, "");		// not specified - assume server is local machine
		printf("Connecting to server at LocalMachine...");
	}
	// connect
	retCode = theClient.Initialize(szMyIPAddress, szServerIPAddress);
	if (retCode != 0)
	{
		printf("Unable to connect to server. Exiting");
		return 1;
	}
	else
	{
		HostInfo hostInfo;
		memset(&hostInfo, 0, sizeof(hostInfo));
		theClient.GetHostInfo(&hostInfo);
		if(!hostInfo.FoundHost)
		{
			printf("Unable to connect to server. Exiting.");
			return 1;
		}
		printf("Connected.\nClient IP:%s\nServer IP:%s\n", szMyIPAddress, szServerIPAddress);
		printf("Server Name:%s", hostInfo.szHostComputerName);
	}

	// Ready to receive marker stream!
	printf("\nClient is connected to server and listening for data...\n");
	int c;
	bool bExit = false;
	while(c =_getch())
	{
		switch(c)
		{
			case 'q':
				bExit = true;		
				break;	
			default:
				break;
		}
		if(bExit)
			break;
	}

	// Done - clean up.
	theClient.Exit();

	return 0;
}

// DataHandler receives data from the server
void __cdecl DataHandler(sFrameOfMocapData* data)
{
	printf("Received frame %d\n", data->iFrame);

	// Other Markers
	printf("Other Markers [Count=%d]\n", data->nOtherMarkers);
	for(int i=0; i < data->nOtherMarkers; i++)
	{
		printf("Other Marker %d : %3.2f\t%3.2f\t%3.2f\n",
			i,
			data->OtherMarkers[i][0],
			data->OtherMarkers[i][1],
			data->OtherMarkers[i][2]);
	}

	// Rigid Bodies
	printf("Rigid Bodies [Count=%d]\n", data->nRigidBodies);
	for(int i=0; i < data->nRigidBodies; i++)
	{
		printf("Rigid Body [ID=%d]\n", data->RigidBodies[i].ID);
		printf("\tx\ty\tz\tqx\tqy\tqz\tqw\n");
		printf("\t%3.2f\t%3.2f\t%3.2f\t%3.2f\t%3.2f\t%3.2f\t%3.2f\n",
			data->RigidBodies[i].x,
			data->RigidBodies[i].y,
			data->RigidBodies[i].z,
			data->RigidBodies[i].qx,
			data->RigidBodies[i].qy,
			data->RigidBodies[i].qz,
			data->RigidBodies[i].qw);

		printf("\tRigid body markers [Count=%d]\n", data->RigidBodies[i].nMarkers);
		printf("\tx\ty\tz\n");
		for(int iMarker=0; iMarker < data->RigidBodies[i].nMarkers; iMarker++)
		{
			printf("\t%3.2f\t%3.2f\t%3.2f\n",
				data->RigidBodies[i].Markers[iMarker][0],
				data->RigidBodies[i].Markers[iMarker][1],
				data->RigidBodies[i].Markers[iMarker][2]);
		}
	}
}

// MessageHandler receives NatNet error/debug messages
void __cdecl MessageHandler(int msgType, char* msg)
{
	printf("\n[NatNet Message] %s\n", msg);
}
